
;; What is S2 Array?

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro S_comp,decomp,quad,S2
pf1=0.17677
pf2=0.353

S2=dblarr(4,decomp.n_coeffs,decomp.n_coeffs)
beta=decomp.beta

xx=quad[0,0]/beta/beta
xy=quad[0,1]/beta/beta
yy=quad[1,1]/beta/beta
init_coeffs,s2_entries,deli,delj,s2_coeff,n1_coeff,n2_coeff,xx_coeff,xy_coeff,yy_coeff
n=0
m=0
for m=0, decomp.n_coeffs-1 do begin
    ;; Only transfer power if n+m=even                                         
    if  (decomp.n1[m]+decomp.n2[m]) mod 2 eq 0 then begin
        for n=0, decomp.n_coeffs-1 do begin
            dn1=decomp.n1[n]-decomp.n1[m]
            dn2=decomp.n2[n]-decomp.n2[m]
          if (abs(dn1)+abs(dn2)) mod 2 eq 1 and abs(dn1)+abs(dn2) le 3 $
            then begin
              for ig=0,3 do begin
                  for i_entry=0,s2_entries[ig]-1 do begin
                      if dn1 eq deli[ig,i_entry] and dn2 eq delj[ig,i_entry] then begin
                          fact1=stepfact(decomp.n1[m],dn1)
                          fact2=stepfact(decomp.n2[m],dn2)
                          S2[ig,n,m]=beta*fact1*fact2*$
                            (pf1*(s2_coeff[ig,i_entry]+n1_coeff[ig,i_entry]*decomp.n1[m]+$
                                  n2_coeff[ig,i_entry]*decomp.n2[m])+$
                             pf2*(xx_coeff[ig,i_entry]*xx+xy_coeff[ig,i_entry]*xy+$
                                  yy_coeff[ig,i_entry]*yy))
                      endif
                  endfor
              endfor
          endif
      endfor
  endif
endfor


end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Computes the flexion of an image

pro flexion,decomp,g,dgdx,nmax=nmax

if not keyword_set(nmax) then nmax=decomp.n_max

; nmax is the maximum coefficient to use in the shapelet analysis

quad=shapelets_quadrupole(decomp,flux=flux)
quad=quad/flux

xx=quad[0,0]
xy=quad[0,1]
yy=quad[1,1]
denom=xx+yy+2*sqrt(xx*yy-xy*xy)
g=[(xx-yy)/denom,2.*xy/denom]
print,"About to run S_Comp"
S_comp,decomp,quad,S2

if total(decomp.coeffs_error) lt 1e-5 then decomp.coeffs_error=decomp.coeffs_error+1

print,"Computing A & B:",systime()
B2=dblarr(4)
A2=dblarr(4,4)
n=0
m=0

mu=decomp.coeffs
for i=0,decomp.n_coeffs-1 do begin
    id=where(decomp.n1[i] eq decomp.n2 and decomp.n2[i] eq decomp.n1)
    mu[i]=(decomp.coeffs[i]+decomp.coeffs[id])/2.
    mu[id]=mu[i]
endfor

id_source=where((decomp.n1+decomp.n2) mod 2 eq 0 and decomp.n1+decomp.n2 le nmax+3,n_source)
for i_source=0,n_source-1 do begin
    l=id_source[i_source]

    diff=abs(decomp.n1-decomp.n1[l])+abs(decomp.n2-decomp.n2[l])
    id_sink=where(diff eq 1 or diff eq 3 and decomp.n1+decomp.n2 le nmax,n_sink)

    
    for i=0,3 do begin
        B2[i]=B2[i]+total(S2[i,id_sink,l]*decomp.coeffs[id_sink]*decomp.coeffs[l]*$
                         decomp.coeffs_error[id_sink])
        for j=0,3 do begin
            for i_sink=0,n_sink-1 do begin
                n=id_sink[i_sink]
                diff2=abs(decomp.n1-decomp.n1[n])+abs(decomp.n2-decomp.n2[n])
                id_local=where(diff2 eq 1 or diff2 eq 3,n_local)
                A2[j,i]=A2[j,i]+total(S2[j,n,id_local]*S2[i,n,l]*$
                                      decomp.coeffs[id_local]*decomp.coeffs[l]*$
                                     decomp.coeffs_error[id_local])
            endfor
        endfor
    endfor
endfor

print,"Inverting A & B:",systime()
dgdx=invert(A2)#B2


print,"Done with flexion:",systime()

end 

