pro psf_test

; Take an image, lens it, and convolve with a known PSF.  How do we
; do?

;; Set up the general lensing parameters...
kappa=0.
gamma1=0.
gamma2=0.
A=[[1-kappa-gamma1,-gamma2],[-gamma2,1-kappa+gamma1]]

g11=0.004
g12=0.004
g21=0.0
g22=0.0

F_model=[g11+g22,g21-g12]
G_model=[g11-g22,g21+g12]
print,'Model gam=',gamma1,gamma2
print,'Model F=',F_model
print,'Model G=',G_model

D=dblarr(2,2,2)

D[*,*,0]=[[-2*g11-g22,-g21],[-g21,-g22]]
D[*,*,1]=[[-g21,-g22],[-g22,2*g12-g21]]

; Begin by creating a (for now) circular Gaussian


;; Set up the image parameters.
sigma=3.
rmax=6*sigma
nx=fix(2.4*rmax)
xc=nx/2.
yc=nx/2.


img_source=dblarr(nx,nx)*0
img_lensed=dblarr(nx,nx)*0

fmax=100.

eps=0.
bg=200.
noisefrac=.1

;; Lay down the image
for i=0,nx-1 do begin
    for j=0,nx-1 do begin
        r=sqrt(((i-xc)/(1.+eps))^2+((j-yc)/(1.-eps))^2)
        if (r lt rmax) then begin
            img_source[i,j]=fmax*exp(-(r^2/2/sigma^2))
        endif
    endfor
endfor

img_lensed=lens_image(img_source,A,D)

; Now, lens it.

sig_psf=3.
img_smeared=psf_convolve(img_lensed,sig_psf,psf)


;; Finally, add noise
img_smeared=img_smeared+noisefrac*sqrt(img_smeared+bg)*randomn(seed,nx*nx)

window,1,xsize=400,ysize=440,title='smeared'
plt_image,img_smeared,/colbar,/frame


;; This is what we'd ostensibly observe from the groun.  Here's where
;; the analysis begins.

rhalf=halflight(img_smeared,x0=x0)

factor=1.0;
print,'Factor=',factor

flex_factor=psf_correct(img_smeared,psf)
print,'Flex_factor=',flex_factor

; compute the weighting function
weightfunc,img_smeared,factor,w,sig2,rhalf=rhalf

flexion_moments,img_smeared,gam_moments,F_moments,G_moments,w,sig2
print,'Uncorrected Estimates='
print,'Gamma=',gam_moments
print,'F=',F_moments
print,'G=',G_moments

print,'Corrected Estimates='
print,'F_corr=',F_moments*flex_factor[0]
print,'G_corr=',G_moments*flex_factor[1]

;; Now do shapelet analysis
i0=lonarr(nx*nx)
j0=lonarr(nx*nx)
for i=long(0),nx*nx-1 do begin
    i0[i]=i mod nx
    j0[i]=i/nx
endfor
flux=total(img_smeared)
xc=total(i0*img_smeared)/flux
yc=total(j0*img_smeared)/flux
Q11=total((i0-xc)^2*img_smeared)/flux
Q22=total((j0-yc)^2*img_smeared)/flux

theta_min=0.6
theta_max=1.8*sqrt(Q11+Q22)
beta=sqrt(theta_min*theta_max)
n_max=fix(theta_max/theta_min)-1

shapelets_decomp,img_smeared,beta,n_max,decomp,recomp,sky=0,x0=[xc,yc]
x0=shapelets_centroid(decomp)
shapelets_decomp,img_smeared,beta,n_max,decomp,recomp,sky=0,x0=x0

window,2,xsize=400,ysize=440,title='recomp'
plt_image,recomp,/colbar,/frame

;flexion_trunc,decomp,g,dgdx,nmax=5
flexion,decomp,g,dgdx
F_shapelets=[dgdx[0]+dgdx[3],dgdx[2]-dgdx[1]]
G_shapelets=[dgdx[0]-dgdx[3],dgdx[2]+dgdx[1]]
print,'Shapelets'
print,'F=',F_shapelets
print,'G=',G_shapelets

print,'Truncated flexion code'
flexion_trunc,decomp,g,dgdx,nmax=3
F_shapelets=[dgdx[0]+dgdx[3],dgdx[2]-dgdx[1]]
G_shapelets=[dgdx[0]-dgdx[3],dgdx[2]+dgdx[1]]
print,'Shapelets'
print,'F=',F_shapelets
print,'G=',G_shapelets

end
