pro test_flexion

kappa=0.
gamma1=0.
gamma2=0.02
A=[[1-kappa-gamma1,-gamma2],[-gamma2,1-kappa+gamma1]]

g11=0.004
g12=0.004
g21=0.0
g22=0.0

F_model=[g11+g22,g21-g12]
G_model=[g11-g22,g21+g12]
print,'Model gam=',gamma1,gamma2
print,'Model F=',F_model
print,'Model G=',G_model

D=dblarr(2,2,2)

D[*,*,0]=[[-2*g11-g22,-g21],[-g21,-g22]]
D[*,*,1]=[[-g21,-g22],[-g22,2*g12-g21]]

; Begin by creating a (for now) circular Gaussian

niter=20
nfactor=12
dfactor=0.2
factormin=0.4

F1_arr=dblarr(nfactor,niter)
F2_arr=dblarr(nfactor,niter)
G1_arr=dblarr(nfactor,niter)
G2_arr=dblarr(nfactor,niter)

F1_shape_arr=dblarr(niter)
F2_shape_arr=dblarr(niter)
G1_shape_arr=dblarr(niter)
G2_shape_arr=dblarr(niter)

;; Set up the image parameters.
sigma=3.
rmax=6*sigma
nx=fix(2.4*rmax)
xc=nx/2.
yc=nx/2.


img_source=dblarr(nx,nx)*0
img_lensed=dblarr(nx,nx)*0

fmax=100.
eps=0.2
bg=200.
noisefrac=.02

;; Lay down the image
for i=0,nx-1 do begin
    for j=0,nx-1 do begin
        r=sqrt(((i-xc)/(1.+eps))^2+((j-yc)/(1.-eps))^2)
        if (r lt rmax) then begin
            img_source[i,j]=fmax*exp(-((r/sigma)^1.5)/2.)
        endif
    endfor
endfor


for iter=0,19 do begin
    img_lensed=lens_image(img_source,A,D)
    img_lensed=img_lensed+randomn(iseed,nx*nx)*noisefrac*sqrt(img_lensed+bg)
    
    ;window,0,xsize=400,ysize=440
    ;plt_image,img_lensed,/colbar,/frame
    
; compute the weighting function
    for ifactor=0,nfactor-1 do begin
        factor=factormin+ifactor*dfactor
        rhalf=halflight(img_lensed,x0=x0)
        weightfunc,img_lensed,factor,w,sig2,rhalf=rhalf
        
        flexion_moments,img_lensed,gam_moments,F_moments,G_moments,w,sig2
        print,'HOLIfactors Estimates='
        print,'Gamma=',gam_moments
        print,'F=',F_moments
        print,'G=',G_moments
        F1_arr[ifactor,iter]=F_moments[0]/F_model[0]-1
        F2_arr[ifactor,iter]=F_moments[1]/F_model[1]-1.
        G1_arr[ifactor,iter]=G_moments[0]/G_model[0]-1.
        G2_arr[ifactor,iter]=G_moments[1]/G_model[1]-1.
    endfor
        
; Now do shapelets
    i0=lonarr(nx*nx)
    j0=lonarr(nx*nx)
    for i=long(0),nx*nx-1 do begin
        i0[i]=i mod nx
        j0[i]=i/nx
    endfor
    flux=total(img_lensed)
    xc=total(i0*img_lensed)/flux
    yc=total(j0*img_lensed)/flux
    Q11=total((i0-xc)^2*img_lensed)/flux
    Q22=total((j0-yc)^2*img_lensed)/flux
    
    theta_min=0.6
    theta_max=1.8*sqrt(Q11+Q22)
    beta=sqrt(theta_min*theta_max)
    n_max=fix(theta_max/theta_min)-1
    shapelets_decomp,img_lensed,beta,n_max,decomp,recomp,sky=0,x0=[xc,yc]
    x0=shapelets_centroid(decomp)
    shapelets_decomp,img_lensed,beta,n_max,decomp,recomp,sky=0,x0=x0
    
                                ;window,2,xsize=400,ysize=440,title='recomp'
                                ;plt_image,recomp,/colbar,/frame
    
    flexion_trunc,decomp,g,dgdx,nmax=5
    F_shapelets=[dgdx[0]+dgdx[3],dgdx[2]-dgdx[1]]
    G_shapelets=[dgdx[0]-dgdx[3],dgdx[2]+dgdx[1]]
    print,'Shapelets'
    print,'F=',F_shapelets
    print,'G=',G_shapelets
    F1_shape_arr[iter]=F_shapelets[0]/F_model[0]-1.
    F2_shape_arr[iter]=F_shapelets[1]/F_model[1]-1.
    G1_shape_arr[iter]=G_shapelets[0]/G_model[0]-1.
    G2_shape_arr[iter]=G_shapelets[1]/G_model[1]-1.
endfor

factorarr=factormin+dfactor*indgen(nfactor)
meanF=dblarr(nfactor)
meanG=dblarr(nfactor)
stdevF=dblarr(nfactor)
stdevG=dblarr(nfactor)
for ifactor=0,nfactor-1 do begin
    meanF[ifactor]=mean([F1_arr[ifactor,*],F2_arr[ifactor,*]])
    meanG[ifactor]=mean([G1_arr[ifactor,*],G2_arr[ifactor,*]])
    stdevF[ifactor]=stdev([F1_arr[ifactor,*],F2_arr[ifactor,*]])
    stdevG[ifactor]=stdev([G1_arr[ifactor,*],G2_arr[ifactor,*]])
endfor

set_plot,'ps'

device,file='meanF.ps',/color,bits=8
;window,0,title='Fractional Error in Mean Measure'
plot,factorarr,meanF,yrange=[-(max(abs([meanF,meanG]))),max(abs([meanF,meanG]))],color=0,background=-1,thick=2,$
  xtitle='C_W',ytitle='(<F>-Ftrue)/Ftrue, (<G>-Gtrue)/Gtrue'
oplot,factorarr,meanG,linestyle=2,color=0,thick=2
device,/close

device,file='meanG.ps',/color,bits=8
;window,1,title='Standard Deviation in Fractional Error'
plot,factorarr,stdevF,yrange=[0,max(abs([stdevF,stdevG]))],color=0,background=-1,thick=2,$
  xtitle='C_W',ytitle='Stdev. F/Ftrue, G/Gtrue'
oplot,factorarr,stdevG,linestyle=2,color=0,thick=2
device,/close

print,'Shapelet mean=',mean([F1_shape_arr,F2_shape_arr]),mean([G1_shape_arr,G2_shape_arr])
print,'Shapelet stdev=',stdev([F1_shape_arr,F2_shape_arr]),stdev([G1_shape_arr,G2_shape_arr])


read,idum

end
